////////////////////////////////////////////////////////////////////////////////
// Copyright (c) 2011-2019 Magewell Electronics Co., Ltd. (Nanjing)
// All rights reserved.
////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <curl/curl.h>

#define HOST_NAME				"192.168.1.206:80"
#define LOGIN_METHOD			"/usapi?method=login"
#define LOGIN_PARAMS			"&id=Admin&pass=e3afed0047b08059d0fada10f400c1e5"
#define GET_INFO_METHOD			"/usapi?method=get-info"
#define UPLOAD_FIRMWARE_METHOD	"/usapi?method=upload-update-file"
#define FIRMWARE_FILE_NAME		"ultra_encode_hdmi_rev_a_1_3_328.mwf"

static char LOGIN_URL[1024]           = "http://" HOST_NAME LOGIN_METHOD LOGIN_PARAMS;
static char GET_INFO_URL[1024]		  = "http://" HOST_NAME GET_INFO_METHOD;
static char UPLOAD_FIRMWARE_URL[1024] = "http://" HOST_NAME FIRMWARE_FILE_NAME;

typedef struct _stream_buffer_t
{
	char   *buffer;
	size_t size;
	size_t capacity;
} stream_buffer_t;

static size_t curl_write_data_cb(void *buffer, size_t size, size_t nmemb, void *param)
{
	size_t realsize = size * nmemb;
	stream_buffer_t *stream = (stream_buffer_t *)param;

	size_t capacity = (stream->size + realsize + 1);
	if (capacity > stream->capacity)
	{
		char *ptr = (char*)realloc(stream->buffer, capacity);
		stream->buffer = ptr;
		stream->capacity = capacity;

		//printf("realloc memory: %ld\n", capacity);
	}

	memcpy(&(stream->buffer[stream->size]), buffer, realsize);
	stream->size += realsize;
	stream->buffer[stream->size] = 0;

	return realsize;
}

int main(int argc, const char * argv[])
{
	CURLcode res = CURLE_OK;

	CURL *curl = NULL;
	struct curl_slist *slist = NULL;
	struct curl_httppost *formpost = NULL;
	struct curl_httppost *lastptr = NULL;

	size_t res_buffer_capacity = 1024;
	stream_buffer_t res_buffer = {
		/*.buffer =*/ (char*)malloc(res_buffer_capacity),
		/*.size =*/ 0,
		/*.capacity =*/ res_buffer_capacity
	};

	////////////////////////////////////////////////////////////////////////////////
	// usage
#ifdef _WIN32
	printf("usage:\nultra_stream_curl.exe <hostip:port>\n");
#else
	printf("usage:\n./ultra_stream_curl <hostip:port>\n");
#endif
	if (argc > 1) {
		sprintf(LOGIN_URL, "http://%s%s%s", argv[1], LOGIN_METHOD, LOGIN_PARAMS);
		sprintf(GET_INFO_URL, "http://%s%s", argv[1], GET_INFO_METHOD);
		sprintf(UPLOAD_FIRMWARE_URL, "http://%s%s", argv[1], UPLOAD_FIRMWARE_METHOD);
	}

	////////////////////////////////////////////////////////////////////////////////
	// curl init
	{
		curl_global_init(CURL_GLOBAL_ALL);

		curl = curl_easy_init();
		if (NULL == curl) {
			printf("curl_easy_init failed: %s\n", curl_easy_strerror(res));
			curl_global_cleanup();
			return -1;
		}

		curl_easy_setopt(curl, CURLOPT_TIMEOUT, 20L);
		curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, 10L);
		slist = curl_slist_append(slist, "Connection:keep-alive");
		curl_easy_setopt(curl, CURLOPT_HTTPHEADER, slist);
		curl_easy_setopt(curl, CURLOPT_COOKIEJAR, ""/*"pro_convert_cookies.txt"*/);  //Just using this option enables cookies for this session
		//but would not like to save cookie jar file
		curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, curl_write_data_cb);
		curl_easy_setopt(curl, CURLOPT_WRITEDATA, (void *)&res_buffer);
	}

	////////////////////////////////////////////////////////////////////////////////
	// ultra stream login
	{
		printf("\n***** 1. login *****\n");

		curl_easy_setopt(curl, CURLOPT_URL, LOGIN_URL);
		res_buffer.size = 0;
		res_buffer.buffer[res_buffer.size] = 0;

		res = curl_easy_perform(curl);
		if (CURLE_OK == res) {
			printf("login response data:\n%s\n", res_buffer.buffer);
		}
		else {
			printf("login failed: %s\n", curl_easy_strerror(res));
		}
	}

	////////////////////////////////////////////////////////////////////////////////
	// ultra stream get info
	{
		printf("\n***** 2. get info *****\n");

		curl_easy_setopt(curl, CURLOPT_URL, GET_INFO_URL);
		res_buffer.size = 0;
		res_buffer.buffer[res_buffer.size] = 0;

		res = curl_easy_perform(curl);
		if (CURLE_OK == res) {
			printf("get info response data:\n%s\n", res_buffer.buffer);
		}
		else {
			printf("get info failed: %s\n", curl_easy_strerror(res));
		}
	}

	////////////////////////////////////////////////////////////////////////////////
	// ultra stream upload firmware
	{
		printf("\n***** 3. upload firmware *****\n");
		curl_formadd(&formpost, &lastptr,
			CURLFORM_COPYNAME, "file",
			CURLFORM_FILE, FIRMWARE_FILE_NAME,
			CURLFORM_END);
		curl_easy_setopt(curl, CURLOPT_HTTPPOST, formpost);
		curl_easy_setopt(curl, CURLOPT_URL, UPLOAD_FIRMWARE_URL);
		res_buffer.size = 0;
		res_buffer.buffer[res_buffer.size] = 0;

		res = curl_easy_perform(curl);
		if (CURLE_OK == res) {
			printf("upload firmware response data:\n%s\n", res_buffer.buffer);
		}
		else {
			printf("upload firmware failed: %s\n", curl_easy_strerror(res));
		}
	}

	////////////////////////////////////////////////////////////////////////////////
	// curl cleanup
	{
		curl_easy_cleanup(curl);
		curl = NULL;

		if (formpost) {
			curl_formfree(formpost);
			formpost = NULL;
		}

		if (slist) {
			curl_slist_free_all(slist);
			slist = NULL;
		}

		if (res_buffer.buffer) {
			free(res_buffer.buffer);
			res_buffer.buffer = NULL;
		}
		res_buffer.size = 0;
		res_buffer.capacity = 0;

		curl_global_cleanup();
	}

#ifdef _WIN32
	printf("Press \"Enter\" key to exit...\n");
	getchar();
#endif

	return 0;
}
